/*

Envelope
--------
     a0   a1   a2   a3   a4   a5   a6   a7 always 0
    /|\   .   /|\   .   /|\   .   /|\    .
   / | \  .  / | \  .  / | \  .  / | \   .
  /  |  \ . /  |  \ . /  |  \ . /  |  \  .
 /   |   \./   |   \./   |   \./   |   \ . 
/    |    |    |    |    |    |    |    \.
0 t0 | t1 | t2 | t3 | t4 | t5 | t6 | t7 0

ADSR comparison
     /|\
    / | \
   /  |  \__________________________
  /   |   |                         |\
 /    |   |                         | \
/     |   |                         |  \
   A    D              S              R

t0 = attack
t1 = decay
t2 - t6 = sustain
t7 = release

A & D alway trigger irrespective of the Gate length

DC*_a0..DC*_a6 : amplitude (0-255) (a7 always 0)
DC*_t0..DC*_t7 : time to change between consequtive amplitude points (0-8000ms)
DC*_jump_from  : sustain loop (1..6)
DC*_jump_to    : sustain loop (0..5)

DC*_s0..DC*_s7 : step size (0..65535)

DC*_s calculation:
-----------------
DC*_s[] = ((1/engine frequency) / slope time) * (delta * 256)
where
   engine frequency in Hz
   slope time = DC*_t[DC*_ptr] in seconds
   delta = DC*_a[DC*_ptr] - DC*_a[DC*_ptr - 1]

simplified:
DC*_s[] = (EnvConst/slope time) * delta
where
   EnvConst = (1/engine frequency) * 256 * 1000
   slope time in ms
   delta = DC*_a[DC*_ptr] - DC*_a[DC*_ptr - 1]
*/

//GLOBAL
//int1 KeyDown = 0;
//int16 const engine_frequency = 1000;
//int8 const envelope_prescale_count = 32;

//int8 Const parked = 255;




// DCA /////////////////////////////////////////////////////////////////////////

int16 DCA_now   = 0;
int16 DCA_next  = 0;
int16 DCAenv_A  = 0;      // Attack step size  (0 - 65535) 
int16 DCAenv_D  = 0xFFFF; // Attack step size  (0 - 65535) - not used in this instance - hardcoded to 0xFFFF
int8  DCAenv_S  = 255;    // Sustain level     (0 - 255)   - not used in this instance - hardcoded to 255  
int16 DCAenv_R  = 0;      // Release step size (0 - 65535)
int8 DCAenv_ptr = 0;

int16 DCF_now   = 0;
int16 DCF_next  = 0;
int16 DCFenv_A  = 0;      // Attack step size  (0 - 65535) 
int16 DCFenv_D  = 0xFFFF; // Attack step size  (0 - 65535) - not used in this instance - hardcoded to 0xFFFF
int8  DCFenv_S  = 255;    // Sustain level     (0 - 255)
int16 DCFenv_R  = 0;      // Release step size (0 - 65535)
int8 DCFenv_ptr = 0;

//int8 DCFRes_now  = 0;
//int8 DCFRes_next = 0;

//------------------------------------------------------------------------------

void DCAenv_thread(){
int16 DCAenv_target = 0;
int16 DCAenv_temp;
 
  switch (DCAenv_ptr){
      case 0  : goto DCAenv_00; // envelope idle
      case 1  : goto DCAenv_01; // Attack
      case 2  : goto DCAenv_02; // Decay
      case 3  : goto DCAenv_03; // Sustain
      case 4  : goto DCAenv_04; // Release
   }
   
DCAenv_00: // envelope idle
   DCAenv_ptr = 0;
   goto DCAenv_done;

DCAenv_01: // Attack   
  
   DCAenv_target = make16(255, 0);
   if (DCA_next < DCAenv_target){
   
      DCAenv_temp = DCA_next + DCAenv_A;
      if (DCAenv_temp > DCAenv_target)DCA_next = DCAenv_target;
      else {
         if (DCAenv_temp < DCA_next) DCA_next = DCAenv_target; // overflow
         else DCA_next = DCAenv_temp;
      }
   }         
   else DCAenv_ptr = 2;
   if (!Gate) DCAenv_ptr = 2;    
   goto DCAenv_done;

DCAenv_02: // Decay
   DCAenv_target = make16(DCAenv_S, 0);
   if (DCA_next > DCAenv_target){
      DCAenv_temp = DCA_next - DCAenv_D;
      if (DCAenv_temp < DCAenv_target) DCA_next = DCAenv_target;
       else{
         if (DCAenv_temp > DCA_next) DCA_next = DCAenv_target; // overflow
         else DCA_next = DCAenv_temp;
       }            
   }         
   else DCAenv_ptr = 3;
   goto DCAenv_done;

DCAenv_03: // Sustain
   if (!env_amp_sust){DCAenv_ptr = 4; goto DCAenv_04;}
   if (Gate) DCAenv_ptr = 3;
   else DCAenv_ptr = 4;
   goto DCAenv_done;

DCAenv_04: // Release
   DCAenv_target = 0;
   if (DCA_next > DCAenv_target){
      DCAenv_temp = DCA_next - DCAenv_R;
      if (DCAenv_temp < DCAenv_target) DCA_next = DCAenv_target;
       else{
         if (DCAenv_temp > DCA_next) DCA_next = DCAenv_target;
         else DCA_next = DCAenv_temp;
       }            
   }         
   else DCAenv_ptr = 0; // envelope done

   goto DCAenv_done;

DCAenv_done:
;
}

//------------------------------------------------------------------------------

void DCAenv_start(){
   DCAenv_ptr = 1;
   DCA_next = 0;
}

//------------------------------------------------------------------------------

void DCFenv_thread(){
int16 DCFenv_target = 0;
int16 DCFenv_temp;

  switch (DCFenv_ptr){
      case 0  : goto DCFenv_00; // envelope idle
      case 1  : goto DCFenv_01; // Attack
      case 2  : goto DCFenv_02; // Decay
      case 3  : goto DCFenv_03; // Sustain
      case 4  : goto DCFenv_04; // Release
   }
   
DCFenv_00: // envelope idle
   DCFenv_ptr = 0;
   goto DCFenv_done;

DCFenv_01: // Attack      
   DCFenv_target = make16(DCFenv_S, 0);
   if (DCF_next < DCFenv_target){
      DCFenv_temp = DCF_next + DCFenv_A;
      if (DCFenv_temp > DCFenv_target)DCF_next = DCFenv_target;
      else {
         if (DCFenv_temp < DCF_next) DCF_next = DCFenv_target; // overflow
         else DCF_next = DCFenv_temp;
      }
   }         
   else DCFenv_ptr = 3;
   if (!Gate)DCFenv_ptr = 3;
   LFOphase = make16(190, 0);;
   DCF_LFO_now = make8(DCF_next, 1);   
   goto DCFenv_done;

DCFenv_02: // Decay
   DCFenv_target = make16(DCFenv_S, 0);
   if (DCF_next > DCFenv_target){
      DCFenv_temp = DCF_next - DCFenv_D;
      if (DCFenv_temp < DCFenv_target) DCF_next = DCFenv_target;
       else{
         if (DCFenv_temp > DCF_next) DCF_next = DCFenv_target; // overflow
         else DCF_next = DCFenv_temp;
       }            
   }         
   else DCFenv_ptr = 3;
   DCF_LFO_now = make8(DCF_next, 1);
   goto DCFenv_done;

DCFenv_03: // Sustain
//LFOphase = 0;
   if (!env_filt_sust){DCFenv_ptr = 4; goto DCFenv_04;}
   DCF_next = make16(DCFenv_S, 0);
   if (Gate) DCFenv_ptr = 3;
   else {
      DCFenv_ptr = 4;
      DCF_next = make16(DCF_LFO_next, 0); //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   }
   DCF_LFO_now = DCF_LFO_next;
   

   goto DCFenv_done;

DCFenv_04: // Release
   DCFenv_target = 0;
   if (DCF_next > DCFenv_target){
      DCFenv_temp = DCF_next - DCFenv_R;
      if (DCFenv_temp < DCFenv_target) DCF_next = DCFenv_target;
       else{
         if (DCFenv_temp > DCF_next) DCF_next = DCFenv_target;
         else DCF_next = DCFenv_temp;
       }            
   }         
   else DCFenv_ptr = 0; // envelope done
   DCF_LFO_now = make8(DCF_next, 1);
   goto DCFenv_done;

DCFenv_done:
;
}

//------------------------------------------------------------------------------

void DCFenv_start(){
   DCFenv_ptr = 1;
   DCF_next = 0;
}

//------------------------------------------------------------------------------
